package com.sun.org.apache.xml.internal.security.keys.keyresolver;

import java.util.concurrent.CopyOnWriteArrayList;
import com.sun.org.slf4j.internal.LoggerFactory;
import javax.crypto.SecretKey;
import com.sun.org.apache.xml.internal.security.keys.keyresolver.implementations.ECKeyValueResolver;
import com.sun.org.apache.xml.internal.security.keys.keyresolver.implementations.X509DigestResolver;
import com.sun.org.apache.xml.internal.security.keys.keyresolver.implementations.KeyInfoReferenceResolver;
import com.sun.org.apache.xml.internal.security.keys.keyresolver.implementations.DEREncodedKeyValueResolver;
import com.sun.org.apache.xml.internal.security.keys.keyresolver.implementations.X509IssuerSerialResolver;
import com.sun.org.apache.xml.internal.security.keys.keyresolver.implementations.X509SubjectNameResolver;
import com.sun.org.apache.xml.internal.security.keys.keyresolver.implementations.RetrievalMethodResolver;
import com.sun.org.apache.xml.internal.security.keys.keyresolver.implementations.X509SKIResolver;
import com.sun.org.apache.xml.internal.security.keys.keyresolver.implementations.X509CertificateResolver;
import com.sun.org.apache.xml.internal.security.keys.keyresolver.implementations.DSAKeyValueResolver;
import com.sun.org.apache.xml.internal.security.keys.keyresolver.implementations.RSAKeyValueResolver;
import java.util.ArrayList;
import com.sun.org.apache.xml.internal.security.utils.JavaUtils;
import java.security.PublicKey;
import java.util.Iterator;
import java.security.cert.X509Certificate;
import com.sun.org.apache.xml.internal.security.keys.storage.StorageResolver;
import org.w3c.dom.Element;
import java.util.List;
import com.sun.org.slf4j.internal.Logger;

public class KeyResolver
{
    private static final Logger LOG;
    private static List<KeyResolver> resolverVector;
    private final KeyResolverSpi resolverSpi;
    
    private KeyResolver(KeyResolverSpi resolverSpi) {
        this.resolverSpi = resolverSpi;
    }
    
    public static int length() {
        return KeyResolver.resolverVector.size();
    }
    
    public static final X509Certificate getX509Certificate(Element element, String s, StorageResolver storageResolver) throws KeyResolverException {
        for (KeyResolver keyResolver : KeyResolver.resolverVector) {
            if (keyResolver == null) {
                throw new KeyResolverException("utils.resolver.noClass", new Object[] { (element != null && element.getNodeType() == 1) ? element.getTagName() : "null" });
            }
            KeyResolver.LOG.debug("check resolvability by class {}", new Object[] { keyResolver.getClass() });
            X509Certificate resolveX509Certificate = keyResolver.resolveX509Certificate(element, s, storageResolver);
            if (resolveX509Certificate != null) {
                return resolveX509Certificate;
            }
            continue;
        }
        throw new KeyResolverException("utils.resolver.noClass", new Object[] { (element != null && element.getNodeType() == 1) ? element.getTagName() : "null" });
    }
    
    public static final PublicKey getPublicKey(Element element, String s, StorageResolver storageResolver) throws KeyResolverException {
        for (KeyResolver keyResolver : KeyResolver.resolverVector) {
            if (keyResolver == null) {
                throw new KeyResolverException("utils.resolver.noClass", new Object[] { (element != null && element.getNodeType() == 1) ? element.getTagName() : "null" });
            }
            KeyResolver.LOG.debug("check resolvability by class {}", new Object[] { keyResolver.getClass() });
            PublicKey resolvePublicKey = keyResolver.resolvePublicKey(element, s, storageResolver);
            if (resolvePublicKey != null) {
                return resolvePublicKey;
            }
            continue;
        }
        throw new KeyResolverException("utils.resolver.noClass", new Object[] { (element != null && element.getNodeType() == 1) ? element.getTagName() : "null" });
    }
    
    public static void register(String s, boolean globalResolver) throws ClassNotFoundException, IllegalAccessException, InstantiationException {
        JavaUtils.checkRegisterPermission();
        KeyResolverSpi keyResolverSpi = (KeyResolverSpi)ClassLoaderUtils.loadClass(s, KeyResolver.class).newInstance();
        keyResolverSpi.setGlobalResolver(globalResolver);
        register(keyResolverSpi, false);
    }
    
    public static void registerAtStart(String s, boolean globalResolver) {
        JavaUtils.checkRegisterPermission();
        ReflectiveOperationException cause = null;
        try {
            KeyResolverSpi keyResolverSpi = (KeyResolverSpi)ClassLoaderUtils.loadClass(s, KeyResolver.class).newInstance();
            keyResolverSpi.setGlobalResolver(globalResolver);
            register(keyResolverSpi, true);
        }
        catch (ClassNotFoundException ex) {
            cause = ex;
        }
        catch (IllegalAccessException ex2) {
            cause = ex2;
        }
        catch (InstantiationException ex3) {
            cause = ex3;
        }
        if (cause != null) {
            throw (IllegalArgumentException)new IllegalArgumentException("Invalid KeyResolver class name").initCause(cause);
        }
        return;
    }
    
    public static void register(KeyResolverSpi keyResolverSpi, boolean b) {
        JavaUtils.checkRegisterPermission();
        KeyResolver keyResolver = new KeyResolver(keyResolverSpi);
        if (b) {
            KeyResolver.resolverVector.add(0, keyResolver);
        }
        else {
            KeyResolver.resolverVector.add(keyResolver);
        }
    }
    
    public static void registerClassNames(List<String> list) throws ClassNotFoundException, IllegalAccessException, InstantiationException {
        JavaUtils.checkRegisterPermission();
        ArrayList list2 = new ArrayList(list.size());
        Iterator<Object> iterator = list.iterator();
        while (iterator.hasNext()) {
            KeyResolverSpi keyResolverSpi = (KeyResolverSpi)ClassLoaderUtils.loadClass((String)iterator.next(), KeyResolver.class).newInstance();
            keyResolverSpi.setGlobalResolver(false);
            list2.add(new KeyResolver(keyResolverSpi));
        }
        KeyResolver.resolverVector.addAll(list2);
    }
    
    public static void registerDefaultResolvers() {
        ArrayList<KeyResolver> list = new ArrayList();
        list.add(new KeyResolver(new RSAKeyValueResolver()));
        list.add(new KeyResolver(new DSAKeyValueResolver()));
        list.add(new KeyResolver(new X509CertificateResolver()));
        list.add(new KeyResolver(new X509SKIResolver()));
        list.add(new KeyResolver(new RetrievalMethodResolver()));
        list.add(new KeyResolver(new X509SubjectNameResolver()));
        list.add(new KeyResolver(new X509IssuerSerialResolver()));
        list.add(new KeyResolver(new DEREncodedKeyValueResolver()));
        list.add(new KeyResolver(new KeyInfoReferenceResolver()));
        list.add(new KeyResolver(new X509DigestResolver()));
        list.add(new KeyResolver(new ECKeyValueResolver()));
        KeyResolver.resolverVector.addAll(list);
    }
    
    public PublicKey resolvePublicKey(Element element, String s, StorageResolver storageResolver) throws KeyResolverException {
        return this.resolverSpi.engineLookupAndResolvePublicKey(element, s, storageResolver);
    }
    
    public X509Certificate resolveX509Certificate(Element element, String s, StorageResolver storageResolver) throws KeyResolverException {
        return this.resolverSpi.engineLookupResolveX509Certificate(element, s, storageResolver);
    }
    
    public SecretKey resolveSecretKey(Element element, String s, StorageResolver storageResolver) throws KeyResolverException {
        return this.resolverSpi.engineLookupAndResolveSecretKey(element, s, storageResolver);
    }
    
    public void setProperty(String s, String s2) {
        this.resolverSpi.engineSetProperty(s, s2);
    }
    
    public String getProperty(String s) {
        return this.resolverSpi.engineGetProperty(s);
    }
    
    public boolean understandsProperty(String s) {
        return this.resolverSpi.understandsProperty(s);
    }
    
    public String resolverClassName() {
        return this.resolverSpi.getClass().getName();
    }
    
    public static Iterator<KeyResolverSpi> iterator() {
        return new KeyResolver.ResolverIterator(KeyResolver.resolverVector);
    }
    
    static {
        LOG = LoggerFactory.getLogger(KeyResolver.class);
        KeyResolver.resolverVector = new CopyOnWriteArrayList();
    }
    
    static class ResolverIterator implements Iterator<KeyResolverSpi>
    {
        List<KeyResolver> res;
        Iterator<KeyResolver> it;
        
        public ResolverIterator(List<KeyResolver> res) {
            this.res = res;
            this.it = this.res.iterator();
        }
        
        @Override
        public boolean hasNext() {
            return this.it.hasNext();
        }
        
        @Override
        public KeyResolverSpi next() {
            KeyResolver keyResolver = (KeyResolver)this.it.next();
            if (keyResolver == null) {
                throw new RuntimeException("utils.resolver.noClass");
            }
            return keyResolver.resolverSpi;
        }
        
        @Override
        public void remove() {
            throw new UnsupportedOperationException("Can't remove resolvers using the iterator");
        }
    }
}
